from django.test import override_settings
from django.shortcuts import render_to_response
from django.core.paginator import Paginator


def example_view(request, template_name):
    objects = ['foo'] * 100
    paginator = Paginator(objects, 2)

    page = paginator.page(request.GET.get('page', 1))

    use_template = 'django_rangepaginator/%s.html' % template_name
    with override_settings(RANGE_PAGINATOR_TEMPLATE=use_template):
        return render_to_response(
            '%s.html' % template_name, {'page': page, 'request': request})



from django import template

from django.conf import settings
from django.template import loader
from six.moves.urllib.parse import parse_qs, urlencode, urlparse, urlunparse

from django_rangepaginator.layout import calculate_pages

register = template.Library()



@register.simple_tag
def paginate(page=None, request=None, distance=2, edge=1, extra_class='',
             text_labels=True):
    paginator = page.paginator
    pages = calculate_pages(
        page.number, paginator.num_pages, distance=distance, edge=edge)

    prev_page_url = next_page_url = None
    result = []
    if request:
        parts = urlparse(request.get_full_path())
        params = parse_qs(parts.query)
        for page_num in pages:
            if not page_num:
                result.append((page_num, None))
            else:
                params['page'] = [str(page_num)]
                query = urlencode(params, doseq=True)
                url = urlunparse(parts[:4] + (query,) + parts[5:])
                result.append((page_num, url))

        if page.has_previous():
            params['page'] = page.previous_page_number()
            query = urlencode(params, doseq=True)
            prev_page_url = urlunparse(parts[:4] + (query,) + parts[5:])

        if page.has_next():
            params['page'] = page.next_page_number()
            query = urlencode(params, doseq=True)
            next_page_url = urlunparse(parts[:4] + (query,) + parts[5:])

    else:
        for page_num in pages:
            url = '?%s' % urlencode({'page': str(page_num)})
            result.append((page_num, url))

        if page.has_previous():
            prev_page_url = '?%s' % urlencode({
                'page': str(page.previous_page_number())
            })

        if page.has_next():
            next_page_url = '?%s' % urlencode({
                'page': str(page.next_page_number())
            })

    pages = result

    context = {
        'current': page.number,
        'page': page,
        'pages': pages,
        'paginator': paginator,
        'next_page_url': next_page_url,
        'prev_page_url': prev_page_url,
        'extra_class': extra_class,
        'text_labels': text_labels,
    }

    template_name = getattr(
        settings, 'RANGE_PAGINATOR_TEMPLATE',
        'django_rangepaginator/bootstrap3.html')
    template = loader.get_template(template_name)
    return template.render(context)



from django.conf.urls import url

from django.views.generic import TemplateView


from sandbox import views

urlpatterns = [
    url(r'^$', TemplateView.as_view(template_name='index.html')),
    url(r'^bootstrap3$', views.example_view, kwargs={
        'template_name': 'bootstrap3'
    }, name='bootstrap3'),
    url(r'^bootstrap4$', views.example_view, kwargs={
        'template_name': 'bootstrap4'
    }, name='bootstrap4'),
]



def calculate_pages(current, total, distance=2, edge=1):
    center = min(max(distance + 1, current), total - distance)
    add_skip_start = add_skip_end = False
    pages = set()

    # Add the page numbers on the start
    for i in range(1, edge + 1):
        pages.add(i)

    # Add range of the current page
    for i in range(max(1, center - distance), min(center + distance + 1, total)):
        pages.add(i)

    # Add page numbers on the end
    for i in range(total + 1 - edge, total + 1):
        pages.add(i)

    # Check if we need to insert a 'skip' or can just fill in the number. Do
    # this for both the start and the end
    if center - distance > edge:
        if edge + 2 in pages:
            pages.add(edge + 1)
        else:
            add_skip_start = True

    if center + distance < total - edge:
        if center + distance + 2 in pages:
            pages.add(center + distance + 1)
        else:
            add_skip_end = True

    pages = sorted(pages)

    # Insert the skip items
    if add_skip_start:
        pages.insert(edge, None)
    if add_skip_end:
        pages.insert(0 - edge, None)

    return pages
